#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <wordexp.h>
#include "keyvalue.h"

const char version[] = "1.0.6";

/*
 *  TODO:
 *  
 *
 *  Changes:
 *
 *  1.0.6
 *	 o dparm(6) = 6 for spflg
 *	 o sour = '' for spflg
 *
 *  1.0.5a
 *	 o don't TECOR or CLCOR in XCOPY
 *	 o set params for tvout before spflg
 *	 o no averaging time in spflg
 *
 *  1.0.5
 *	 o bundle fg2flg.pl with autoaips
 *	 o allow up to 999 flags now.
 *	 o XCOPY now reruns CLCOR and TECOR since coords are slightly different.
 *		Also copy SN tables and run CLCAL instead of copying CL tables
 *	 o spflg : set parameters for tbout.
 *	 o spflg : now include autocorrelations and do 6 second avg.
 *	 o spflg : first duplicate FG table.  Run spflg on ver 2.
 *	 o revert the "return of autoaips" changes
 *
 *  1.0.4
 *       o add dotv=1 to imager defaults
 *
 *  1.0.3 Aug 20, 2003
 *	 o added inver = 0 to clcal to track aips
 *	 o added some additional default parameters to imagr, calib
 *
 *  1.0.2 Aug 12, 2003
 *
 *	 o set outvers for bpass -- required for recent versions of aips.
 *
 *  1.0.1  Aug 7, 2003
 *
 *	 o add gainuse to clcor to keep up with aips changes
 *	 o possm params change : aparm(9)=1, nplot=2
 *
 *  1.0.0  Dec 17, 2002
 *
 *	 o Rename it!
 *
 *  0.9.9k Dec 03, 2002
 *
 *	 o in SNPLT use factor = 0.2 to make plots more legible
 *	 o set outdisk in calib
 *	 o "TASK" spflg, tacop
 *
 *  0.9.9j Dec 02, 2002
 *
 *	 o fixed logic in determining data path
 *
 *  0.9.9i Nov 25, 2002
 *	 o MODEL --> MODEL<n>
 *	 o GEOD  --> GEOD<n>
 *	 o Do exhaustive baseline search in fringe
 *	 o tune the fringe solint
 *	 o if expcode < bc123az then guess the epoch  (< bc123ai is 1)
 *	 o omit chan 1,2 and 31,32 when doing 15 minute amp self cal
 *
 *  0.9.9h Nov 22, 2002
 *
 *	 o fix core dump if Env Var not defined
 *	 o cleaning up output text
 *	 o allows (and suggests) use of .x.<n> (n = epoch number) files
 *		over just .x files
 *	 o grab three days of jpl tecor data
 *
 *  0.9.9g Sep 26, 2002 
 *
 *	 o autoaips.howto
 *
 *  0.9.9f Sep 26, 2002  (extra nicities)
 *
 *	 o Default bandpass calibrator is now phase cal #1
 *	 o TPUT good parameters for SPFLG after XAPRIORI
 *	 o TPUT good params for IMAGR, CALIB, TACOP after XCOPY and XPHASE
 *	 o Leave XCALIB files behind, not CALIB, so cleanup doesn't
 *		interfere with other projects
 *	 o XUVFLG -- duplicate of flag section of XAPRIORI
 *	 o message when XCOPY is done and other improved finish messages
 *
 *  0.9.9e Sep 19, 2002  (fixes to 0.9.9d)
 *
 *	 o fix change to ionex.py call
 *	 o use egetname in cleanup and uvcleanup so recat runs
 *	 o logs are now gotten from:
 *   http://www.astro.cornell.edu/~shami/psrvlb/observe/logs/<filename.log>
 *
 *  0.9.9d Sep 16, 2002  (changes suggested by Shami)
 *
 *	 o Documentation update:  *.doc --> *.txt
 *                                update autoaips.txt
 *	 o Clarify comments to user
 *	 o Path additions via environment variables:
 *		IONEXPY can now point to ionex.py if it isn't in exe. path.
 *		BOXFILES can point to global box file directory
 *		DATAFILES can point to the source of data files (ie /mnt/cdrom)
 *	 o add CCMRG after each image is made
 *	 o do CCNTR after each MODEL is made
 *	 o do some date checking to ensure correct .x file is used
 *	 o imager params --> robust = 3, minpatch = 101
 *
 *  0.9.9c Sep 08, 2002
 *
 *  	 o use ib1 as the default in XCOPY
 * 	 o Save temp file with pulsar name
 * 	 o boxfiles auto generated if none present
 *	 o don't make XFLAG -- it never really worked.
 *
 *  0.9.9b Aug 19, 2002
 *
 * 	 o finally nailed that bandpass calibration bug.  Also
 *		revamped possm.
 *
 *  0.9.9a Aug 18, 2002
 *
 *	 o fixed bug introduced with 15 min amp self cal.
 *	 o grab observing log from Shami's web page.
 *	 o bpass solint changed from 1000 to -1 
 *
 *  0.9.9  Aug 12, 2002
 *
 *	 o amp selfcal the phase calibrator
 *	 o run snplt on fringe phase connections
 *       o start each file with clrmsg
 *	 o TVIN before each TV use
 *
 *  0.9.8  Jul 24, 2002
 *
 *	 o add snsmo after fring
 *	 o split only channels 2 through 31 -- the edge channels are
 *		flagged by bpass anyway.  Before BP table is created,
 *		just split out channels 4 through 29.
 *	 o move indxr to XLOAD -- this way XAPRIORI never changes the
 *		initially loaded data.
 *
 *  0.9.7  Jul 22, 2002
 *
 *	 o script to copy tables for in-beams
 *  	 o add apriori flags
 *
 *  0.9.6  Jul 19, 2002
 *
 *       o add geodesy calibrator calibration
 *       o fixed solint bug for calib
 *
 *  0.9.5  Jul 19, 2002
 *
 *       o always run indxr to prevent splitting of scans
 *       o added accor
 *       o minor cleanups
 *
 *  0.9    Jul 17, 2002
 *
 *       o prototype version
 */

struct KeyValue *kv;
char keyfile[1000], expcode[50], classcode[50];
FILE *out, *tablog = 0;
int disk;
int refant = 4;
int aprioriCL = 0, aprioriSN = 0;
const char *bpasscal;
int pass;	/* 0 for pulsar, 1, 2, 3 for inbeam */
const int doampselfcal = 1;

char *aapath,      aalocation[1000];
char *datapath,    datalocation[100];
char *boxfilepath, boxfilelocation[100];
char ionexpy[200];
char model[8], geodext[8];

struct table
{
	char type[3];
	int count;
};

struct table tables[] =
{
	{ "CL", 0 },
	{ "SN", 0 },
	{ "FG", 0 },
	{ "GC", 0 },
	{ "TY", 0 },
	{ "BP", 0 },
	{ "FG", 0 },
	{ "PC", 0 },
	{ "NX", 0 },
	{ "  ", -1 }
};

int tabver(const char *type)
{
	int i;

	for(i = 0; tables[i].count >= 0; i++)
		if(strcmp(tables[i].type, type) == 0)
			return tables[i].count;
	
	fprintf(stderr, "tabver: table type %s not known\n", type);

	return -1;
}

void savetabs(int *tasave)
{
	int i;
	
	for(i = 0; (tasave[i] = tables[i].count) > 0; i++);
}

void settabs(const int *tasave)
{
	int i;

	for(i = 0; (tables[i].count = tasave[i]) > 0; i++);
}

void inctab(const char *type, const char *desc)
{
	char filename[1000];
	int i;
	
	if(!tablog)
	{
		sprintf(filename, "%s/%s.tablog", aapath, expcode);
		tablog = fopen(filename, "w");
		if(!tablog)
		{
			fprintf(stderr, "Cannot open %s for write\n", filename);
			return;
		}
	}

	for(i = 0; tables[i].count >= 0; i++)
		if(strcmp(tables[i].type, type) == 0)
		{
			tables[i].count++;
			fprintf(tablog, "%s%d  : %s\n", type, tables[i].count, 
				desc);
			return;
		}
		
	fprintf(stderr, "inctab: table type %s not known\n", type);
}

void modtab(const char *type, const char *desc)
{
	int i;

	for(i = 0; tables[i].count >= 0; i++)
		if(strcmp(tables[i].type, type) == 0)
		{
			fprintf(tablog, "%s%d* : %s\n", type, tables[i].count, 
				desc);
			return;
		}
		
	fprintf(stderr, "modtab: table type %s not known\n", type);
}
int usage(const char *program)
{
	fprintf(stderr, "autoaips version %s\n\n", version);
	fprintf(stderr, 
		"Usage : %s <env_var_path> { psr | ib1 | ib2 | ... } <disk>"
		" [<data_file_path> [<box_file_path>] ]\n",
		program);

	return 0;
}

void alldata()
{
	fprintf(out, "  timerang = 0\n");
	fprintf(out, "  stokes = \'\'\n");
	fprintf(out, "  bif = 1; eif = 0\n");
	fprintf(out, "  bchan = 1; echan = 0\n");
	fprintf(out, "  antennas = 0; baseline = 0\n");
	fprintf(out, "  freqid = -1\n");
	fprintf(out, "  refant = REF_ANT\n");
}

void init()
{
	fprintf(out, "restore 0\n");
	fprintf(out, "version = \'%s\'\n", aalocation);
	fprintf(out, "PROC STOOPID\n");
	fprintf(out, "  SCALAR AIPSDISK\n");
	fprintf(out, "  SCALAR REF_ANT\n");
	fprintf(out, "  STRING*6 CALXFR\n");
	fprintf(out, "FINISH\n");
	fprintf(out, "*\n* Change the next line to change the aips disk\n");
	fprintf(out, "AIPSDISK = %d\n", disk);
	fprintf(out, "*\n");
	fprintf(out, "* Change the next line to change the reference antenna\n");
	fprintf(out, "REF_ANT = %d\n", refant);
	fprintf(out, "*\n*\n");
	fprintf(out, "outdisk = AIPSDISK; indisk = AIPSDISK\n");
	fprintf(out, "baddisk = AIPSDISK, 0\n");
	fprintf(out, "CALXFR = \'IB1\'\n");
	fprintf(out, "clr2n\n");
	fprintf(out, "recat\n\n");

	fprintf(out, "\n**********  Paste the above into aips to recontinue\n");
	fprintf(out,   "**********  script execution after restarting aips\n");


	inctab("CL", "Given");
	inctab("PC", "Given");
	inctab("TY", "Given");
	inctab("GC", "Given");
	inctab("FG", "Given");
	inctab("NX", "Given");
}

void fitld(const char *fn)
{
	fprintf(out, "\nTASK \'fitld\'\n");
	fprintf(out, "  outd = AIPSDISK\n");
	fprintf(out, "  infile = \'%s\'\n", fn);
	fprintf(out, "  outname = \'%s\'\n", expcode);
	fprintf(out, "  outclass = \'x%s\'\n", classcode);
	fprintf(out, "  outseq = 1\n");
	fprintf(out, "  doconcat = 1\n");
	fprintf(out, "  douvcomp = -1\n");
	fprintf(out, "  clint = 0.1\n");
	fprintf(out, "  ncount = 1\n");
	fprintf(out, "  digicor = 1\n");
	fprintf(out, "  delcor = 1\n");
	fprintf(out, "  wtthresh = 0.2\n");
	fprintf(out, "GO\n");
	fprintf(out, "WAIT \'fitld\'\n");
}

void indxr()
{
	fprintf(out, "\nTASK \'indxr\'\n");
	fprintf(out, "  indisk = AIPSDISK\n");
	fprintf(out, "  inname = \'%s\'\n", expcode);
	fprintf(out, "  inclass = \'x%s\'\n", classcode);
	fprintf(out, "  infile = \'\'\n");
	fprintf(out, "  in2file = \'\'\n");
	fprintf(out, "  inseq = 1\n");
	fprintf(out, "  inext = 'NX'\n");
	fprintf(out, "  prtlev = 0\n");
	fprintf(out, "  inver = 0\n");
	fprintf(out, "  EXTD\n");
	fprintf(out, "  cparm = 10, 0, 0.1, 1, 1, 0\n");
	fprintf(out, "  bparm = 0\n");
	fprintf(out, "GO\n");
	fprintf(out, "WAIT \'indxr\'\n");
	modtab("NX", "INDXR -- to un-split scans");
}

void fxpol()
{
	fprintf(out, "\nTASK \'fxpol\'\n");
	fprintf(out, "  indisk = AIPSDISK\n");
	fprintf(out, "  inname = \'%s\'\n", expcode);
	fprintf(out, "  inclass = \'x%s\'\n", classcode);
	fprintf(out, "  inseq = 1\n");
	fprintf(out, "  outdisk = AIPSDISK\n");
	fprintf(out, "  outname = \'%s\'\n", expcode);
	fprintf(out, "  outclass = \'%s\'\n", classcode);
	fprintf(out, "  outseq = 1\n");
	fprintf(out, "  bandpol = \'\'\n");
	fprintf(out, "  fqtol = -1\n");
	fprintf(out, "GO\n");
	fprintf(out, "WAIT \'fxpol\'\n");
}

void mergecal()
{
	fprintf(out, "\nTASK \'tamrg\'\n");
	fprintf(out, "  indisk = AIPSDISK\n");
	fprintf(out, "  inname = \'%s\'\n", expcode);
	fprintf(out, "  inclass = \'%s\'\n", classcode);
	fprintf(out, "  inseq = 1\n");

	fprintf(out, "  inext = \'GC\'\n");
	fprintf(out, "  inver = %d\n", tabver("GC"));
	inctab("GC", "TAMRG");
	fprintf(out, "  outver = %d\n", tabver("GC"));
	fprintf(out, "  aparm = 1,1,2,1,3,1\n");
	fprintf(out, "  bparm = 1,2,3\n");
	fprintf(out, "  cparm = 0 ; dparm = 0\n");
	fprintf(out, "GO\n");
	fprintf(out, "WAIT tamrg\n");
	
	fprintf(out, "  inext = \'TY\'\n");
	fprintf(out, "  inver = %d\n", tabver("TY"));
	inctab("TY", "TAMRG");
	fprintf(out, "  outver = %d\n", tabver("TY"));
	fprintf(out, "  aparm = 1,1,4,1,5,1,6,1\n");
	fprintf(out, "  bparm = 1,3,4,5,6\n");
	fprintf(out, "  cparm = (0.1 / (24.0 * 60.0 * 60.0)), 0\n");
	fprintf(out, "  dparm = 0\n");
	fprintf(out, "GO\n");
	fprintf(out, "WAIT tamrg\n");
	
	fprintf(out, "  inext = \'PC\'\n");
	fprintf(out, "  inver = %d\n", tabver("PC"));
	inctab("PC", "TAMRG");
	fprintf(out, "  outver = %d\n", tabver("PC"));
	fprintf(out, "  aparm = 1,1,4,1,5,1,6,1\n");
	fprintf(out, "  bparm = 1,3,4,5,6\n");
	fprintf(out, "  cparm = (0.1 / (24.0 * 60.0 * 60.0)), 0\n");
	fprintf(out, "  dparm = 0\n");
	fprintf(out, "GO\n");
	fprintf(out, "WAIT tamrg\n");
}

void accor()
{
	char desc[1000], buf[1000];
	const char *name;
	int i;

	fprintf(out, "\nTASK \'accor\'\n");
	fprintf(out, "  indisk = AIPSDISK\n");
	fprintf(out, "  inname = \'%s\'\n", expcode);
	fprintf(out, "  inclass = \'%s\'\n", classcode);
	fprintf(out, "  inseq = 1\n");
	fprintf(out, "  timerang = 0\n");
	fprintf(out, "  solint = -3\n");
	fprintf(out, "  snver = 0 ; outver = 0\n");
	fprintf(out, "GO\n");
	fprintf(out, "WAIT accor\n");
	inctab("SN", "ACCOR");
	fprintf(out, "\nTASK \'clcal\'\n");
	fprintf(out, "  indisk = AIPSDISK\n");
	fprintf(out, "  inname = \'%s\'\n", expcode);
	fprintf(out, "  inclass = \'%s\'\n", classcode);
	fprintf(out, "  inseq = 1\n");
	alldata();
	desc[0] = '\'';
	desc[1] = 0;
	for(i = 1; i < 6; i++)
	{
		sprintf(buf, "phase_cal_name%d", i);
		name = getKeyValuestring(kv, buf);
		if(!name) break;
		strcat(desc, name);
		strcat(desc, "\',\'");
	}
	for(i = 1; i < 6; i++)
	{
		sprintf(buf, "geod_cal_name%d", i);
		name = getKeyValuestring(kv, buf);
		if(!name) break;
		strcat(desc, name);
		strcat(desc, "\',\'");
	}
	strcat(desc, "\'");
	fprintf(stderr, "accor runs on sources %s\n", desc);
	fprintf(out, "  calsour = %s\n", desc);
	fprintf(out, "  snver = %d\n", tabver("SN"));
	fprintf(out, "  inver = 0\n");
	fprintf(out, "  gainver = %d\n", tabver("CL"));
	sprintf(desc, "CLCAL accor (CL%d x SN%d)", tabver("CL"), tabver("SN"));
	inctab("CL", desc);
	fprintf(out, "  gainuse = %d\n", tabver("CL"));
	fprintf(out, "  opcode = \'\'\n");
	fprintf(out, "  interpol = \'\'\n");
	fprintf(out, "  intparm = 0\n");
	fprintf(out, "  smotype = \'\'\n");
	fprintf(out, "GO\n");
	fprintf(out, "WAIT \'clcal\'\n");
}

void apcal()
{
	char desc[1000];

	fprintf(out, "\nTASK \'apcal\'\n");
	fprintf(out, "  indisk = AIPSDISK\n");
	fprintf(out, "  inname = \'%s\'\n", expcode);
	fprintf(out, "  inclass = \'%s\'\n", classcode);
	fprintf(out, "  inseq = 1\n");
	alldata();
	fprintf(out, "  sources = \'\'\n");
	fprintf(out, "  gcver = 2\n");
	fprintf(out, "  tyver = 2\n");
	inctab("SN", "APCAL");
	fprintf(out, "  snver = %d\n", tabver("SN"));
	fprintf(out, "  aparm = 0\n");
	fprintf(out, "  opcode = \'\'\n");
	fprintf(out, "GO\n");
	fprintf(out, "WAIT \'apcal\'\n");
	fprintf(out, "\nTASK \'clcal\'\n");
	fprintf(out, "  indisk = AIPSDISK\n");
	fprintf(out, "  inname = \'%s\'\n", expcode);
	fprintf(out, "  inclass = \'%s\'\n", classcode);
	fprintf(out, "  inseq = 1\n");
	alldata();
	fprintf(out, "  calsour = \'\'\n");
	fprintf(out, "  snver = %d\n", tabver("SN"));
	fprintf(out, "  inver = 0\n");
	fprintf(out, "  gainver = %d\n", tabver("CL"));
	sprintf(desc, "CLCAL apcal (CL%d x SN%d)", tabver("CL"), tabver("SN"));
	inctab("CL", desc);
	fprintf(out, "  gainuse = %d\n", tabver("CL"));
	fprintf(out, "  opcode = \'\'\n");
	fprintf(out, "  interpol = \'self\'\n");
	fprintf(out, "  intparm = 0\n");
	fprintf(out, "  smotype = \'\'\n");
	fprintf(out, "GO\n");
	fprintf(out, "WAIT \'clcal\'\n");
}

int clcor(int inver)
{
	char desc[1000];
	
	fprintf(out, "\nTASK \'tacop\'\n");
	fprintf(out, "  indisk = AIPSDISK\n");
	fprintf(out, "  inname = \'%s\'\n", expcode);
	fprintf(out, "  inclass = \'%s\'\n", classcode);
	fprintf(out, "  inseq = 1\n");
	fprintf(out, "  outdisk = AIPSDISK\n");
	fprintf(out, "  outname = \'%s\'\n", expcode);
	fprintf(out, "  outclass = \'%s\'\n", classcode);
	fprintf(out, "  outseq = 1\n");
	fprintf(out, "  inext = \'CL\'\n");
	if(inver == 0)
	{
		fprintf(out, "  inver = %d\n", tabver("CL"));
		sprintf(desc, "TACOP CL%d", tabver("CL"));
		inctab("CL", desc);
		fprintf(out, "  outver = %d\n", tabver("CL"));
	}
	else
	{
		fprintf(out, "  inver = %d\n", inver);
		sprintf(desc, "TACOP CL%d", inver);
		fprintf(out, "  outver = %d\n", inver+1);
	}
	fprintf(out, "  ncount = 1\n");
	fprintf(out, "  keyword = \'\'\n");
	fprintf(out, "  keyvalue = 0; keystrng = \'\'\n");
	fprintf(out, "GO\n");
	fprintf(out, "WAIT \'tacop\'\n");

	fprintf(out, "\nTASK \'clcor\'\n");
	fprintf(out, "  indisk = AIPSDISK\n");
	fprintf(out, "  inname = \'%s\'\n", expcode);
	fprintf(out, "  inclass = \'%s\'\n", classcode);
	fprintf(out, "  inseq = 1\n");
	alldata();
	if(inver == 0)
	{
		fprintf(out, "  gainver = %d\n", tabver("CL"));
		fprintf(out, "  gainuse = %d\n", tabver("CL"));
	}
	else
	{
		fprintf(out, "  gainver = %d\n", inver+1);
		fprintf(out, "  gainuse = %d\n", inver+1);
	}
	fprintf(out, "  opcode = \'PANG\'\n");
	fprintf(out, "  clcorprm = 1 0\n");
	fprintf(out, "GO\n");
	fprintf(out, "WAIT \'clcor\'\n");
	modtab("CL", "CLCOR: pang 1 0");

	if(inver == 0) return tabver("CL");
	else return inver+1;
}

int tecor(int inver)
{
	FILE *pin, *in;
	int i, lst, day, year;
	char buf[1000], command[1000];
	
	sprintf(command, "grep \'day\' %s/%s", aapath, keyfile);
	fprintf(stderr, "Executing : %s\n", command);
	pin = popen(command, "r");
	fgets(buf, 999, pin);
	fclose(pin);
	for(i = 0; ;i++)
	{
		if(buf[i] == 0)
		{
			fprintf(stderr, "bad day line\n");
			return -1;
		}
		if(buf[i] == '=') break;
	}
	sscanf(buf+i+1, "%d", &lst);

	if(getKeyValueint(kv, "lstday") < 0)
	{
		fprintf(stderr, "** Warning : no lstday in .x file\n");
	}
	else if(lst != getKeyValueint(kv, "lstday"))
	{
		return -1;
	}
	
	day = ((float)lst/1.00273791 - 6553.91185 - 0.25);

	year = 2002;
	day -= 52274;
	if(day > 365) { year++; day -= 365; }
	if(day > 365) { year++; day -= 365; }
	if(day > 366) { year++; day -= 366; }
	if(day > 365) { year++; day -= 365; }
	if(day > 365) { year++; day -= 365; }
	if(day > 365) 
	{
		fprintf(stderr, "date out of range\n");
		return -1;
	}
	fprintf(stderr, "year = %d, day = %d\n", year, day);
	
	fprintf(out, "\nTASK \'tecor\'\n");
	fprintf(out, "  indisk = AIPSDISK\n");
	fprintf(out, "  inname = \'%s\'\n", expcode);
	fprintf(out, "  inclass = \'%s\'\n", classcode);
	fprintf(out, "  inseq = 1\n");
	alldata();
	fprintf(out, "  infile = \'%s:jplg%03d0.%02di\n", 
		aalocation, day, year%100);
	fprintf(out, "  nfiles = 3\n");
	if(inver == 0)
	{
		fprintf(out, "  gainver = %d\n", tabver("CL"));
		inctab("CL", "TECOR");
		fprintf(out, "  gainuse = %d\n", tabver("CL"));
	}
	else
	{
		fprintf(out, "  gainver = %d\n", inver);
		fprintf(out, "  gainuse = %d\n", inver+1);
	}
	fprintf(out, "  aparm = 1 0\n");
	fprintf(out, "GO\n");
	fprintf(out, "WAIT \'tecor\'\n");

	for(i = 0; i < 3; i++)
	{
		sprintf(buf, "%s/jplg%03d0.%02di", aapath, day, year%100);
		in = fopen(buf, "r");
		if(!in)
		{
			sprintf(command, "cd %s ; %s %d %d\n", aapath, 
				ionexpy, year, day);
			system(command);
		}
		else fclose(in);
		day++;
		if((year % 4 == 0 && day > 366) ||
		   (year % 4 > 0  && day > 365))
		{
			day = 1;
			year++;
		}
	}

	return 0;
}

void getlog()
{
	char command[1000];

	sprintf(command, "cd %s ; wget -nc http://www.astro.cornell.edu/~shami/psrvlb/observe/logs/%s.log", aapath, expcode);
	printf("executing command : %s\n", command);
	system(command);
}

void setupspflg(const char *source)
{
	fprintf(out, "\n");
	fprintf(out, "TASK \'tacop\'\n");
	fprintf(out, "  indisk = AIPSDISK\n");
	fprintf(out, "  inname = \'%s\'\n", expcode);
	fprintf(out, "  inclass = \'%s\'\n", classcode);
	fprintf(out, "  inseq = 1\n");
	fprintf(out, "  outdisk = indisk\n");
	fprintf(out, "  outname = inname\n");
	fprintf(out, "  outclass = inclass\n");
	fprintf(out, "  outseq = inseq\n");
	fprintf(out, "  inext = \'FG\'\n");
	fprintf(out, "  inver = 1\n");
	fprintf(out, "  outver = 2\n");
	fprintf(out, "  ncount = 1\n");
	fprintf(out, "GO\n");
	fprintf(out, "WAIT tacop\n");

	//fprintf(desc, "FG2  : copy of FG1\n");
	
	fprintf(out, "TASK \'spflg\'\n");
	alldata();
	fprintf(out, "  sour = \'\'\n");
	if(tabver("BP") > 0)
	{
		fprintf(out, "  doband = 1\n");
		fprintf(out, "  bpver = %d\n", tabver("BP"));
	}
	else fprintf(out, "  doband = -1\n");
	fprintf(out, "  flagver = 2\n");
	fprintf(out, "  docalib = 2\n");
	fprintf(out, "  gainuse = %d\n", aprioriCL);
	fprintf(out, "  dparm = 0, 1, 0, 0, 0, 6, 0\n");
	fprintf(out, "  smooth = 0\n");
	fprintf(out, "TPUT spflg\n");
	fprintf(out, "\n");
}

void setuptbout(const char *area)
{
	fprintf(out, "\n");
	fprintf(out, "TASK \'tbout\'\n");
	fprintf(out, "  indisk = AIPSDISK\n");
	fprintf(out, "  inname = \'%s\'\n", expcode);
	fprintf(out, "  inclass = \'%s\'\n", classcode);
	fprintf(out, "  inseq = 1\n");
	fprintf(out, "  inext = \'FG\'\n");
	fprintf(out, "  inver = 2\n");
	fprintf(out, "  outfile = \'%s:AA.FLAG\'\n", area);
	fprintf(out, "  docrt = 132\n");
	fprintf(out, "  bcount = 1\n");
	fprintf(out, "  ecount = 0\n");
	fprintf(out, "TPUT tbout\n");
	fprintf(out, "\n");
}

void fring(const char *source, int usemod, double solint, int allsource)
{
	char desc[100];
	
	fprintf(out, "\nTASK \'fring\'\n");
	fprintf(out, "  indisk = AIPSDISK\n");
	fprintf(out, "  inname = \'%s\'\n", expcode);
	fprintf(out, "  inclass = \'%s\'\n", classcode);
	fprintf(out, "  inseq = 1\n");
	alldata();
	fprintf(out, "  calsour = \'%s\'\'\n", source);
	if(tabver("BP") > 0)
	{
		fprintf(out, "  doband = 1\n");
		fprintf(out, "  bpver = %d\n", tabver("BP"));
	}
	else
	{
		fprintf(out, "  doband = -1\n");
	}
	fprintf(out, "  flagver = %d\n", tabver("FG"));
	fprintf(out, "  docalib = 2\n");
	fprintf(out, "  gainuse = %d\n", aprioriCL);
	if(usemod)
	{
		fprintf(out, "  in2disk = AIPSDISK\n");
		fprintf(out, "  in2name = \'%s\'\n", source);
		fprintf(out, "  in2cl = \'%s\'\n", model);
		fprintf(out, "  in2seq = 0\n");
	}
	else
	{
		fprintf(out, "  clr2n\n");
	}
	fprintf(out, "  invers = 0\n");
	fprintf(out, "  clro\n");
	fprintf(out, "  solint = %f\n", solint);
	fprintf(out, "  aparm = 3 0 0 0 0 1 3.7 0 1 0\n");
	fprintf(out, "  dparm = 0 300 30 0\n");
	sprintf(desc, "FRING %s", source);
	inctab("SN", desc);
	fprintf(out, "  snver = %d\n", tabver("SN"));
	fprintf(out, "GO\n");
	fprintf(out, "WAIT \'fring\'\n");

	fprintf(out, "\nTASK \'snsmo\'\n");
	fprintf(out, "  indisk = AIPSDISK\n");
	fprintf(out, "  inname = \'%s\'\n", expcode);
	fprintf(out, "  inclass = \'%s\'\n", classcode);
	fprintf(out, "  inseq = 1\n");
	alldata();
	fprintf(out, "  bparm = 0, 0, 0.33, 0.25, 0\n");
	fprintf(out, "  doblank = 1\n");
	fprintf(out, "  smotype = \'VLBI\'\n");
	fprintf(out, "  cparm = 0\n");
	fprintf(out, "  interpol = \'BOX\'\n");
	fprintf(out, "  invers = %d\n", tabver("SN"));
	sprintf(desc, "SNSMO SN%d", tabver("SN"));
	inctab("SN", desc);
	fprintf(out, "  outvers = %d\n", tabver("SN"));
	fprintf(out, "GO\n");
	fprintf(out, "WAIT \'snsmo\'\n");
	
	fprintf(out, "\nTASK \'clcal\'\n");
	fprintf(out, "  indisk = AIPSDISK\n");
	fprintf(out, "  inname = \'%s\'\n", expcode);
	fprintf(out, "  inclass = \'%s\'\n", classcode);
	fprintf(out, "  inseq = 1\n");
	alldata();
	fprintf(out, "  calsour = \'%s\'\'\n", source);
	if(allsource) fprintf(out, "  sour = \'\'\n");
	else fprintf(out, "  sour = \'%s\'\'\n", source);
	fprintf(out, "  snver = %d\n", tabver("SN"));
	fprintf(out, "  inver = 0\n");
	fprintf(out, "  gainver = %d\n", aprioriCL);
	sprintf(desc, "CLCAL fring %s (CL%d x SN%d)", source, aprioriCL,
		tabver("SN"));
	inctab("CL", desc);
	fprintf(out, "  gainuse = %d\n", tabver("CL"));
	fprintf(out, "  opcode = \'\'\n");
	fprintf(out, "  interpol = \'ambg\'\n");
	fprintf(out, "  intparm = 0\n");
	fprintf(out, "  smotype = \'\'\n");
	fprintf(out, "GO\n");
	fprintf(out, "WAIT \'clcal\'\n");

	fprintf(out, "\nTASK \'snplt\'\n");
	fprintf(out, "  indisk = AIPSDISK\n");
	fprintf(out, "  inname = \'%s\'\n", expcode);
	fprintf(out, "  inclass = \'%s\'\n", classcode);
	fprintf(out, "  inseq = 1\n");
	alldata();
	fprintf(out, "  sour = \'\'\n");
	fprintf(out, "  inext = \'CL\'\n");
	fprintf(out, "  inver = %d\n", tabver("CL"));
	fprintf(out, "  nplot = 8\n");
	fprintf(out, "  optyp = \'phas\'\n");
	fprintf(out, "  xinc = 1\n");
	fprintf(out, "  opcode = \'\'\n");
	fprintf(out, "  xaxis = 0\n");
	fprintf(out, "  cutoff = 0\n");
	fprintf(out, "  grchan = 0\n");
	fprintf(out, "  dotv = 1\n");
	fprintf(out, "  factor = 0.2\n");
	fprintf(out, "TVIN\n");
	fprintf(out, "GO\n");
	fprintf(out, "WAIT \'snplt\'\n");

	/* amp self cal over 15 minutes */
	if(usemod && doampselfcal)
	{
		fprintf(out, "\nTASK \'calib\'\n");
		fprintf(out, "  indisk = AIPSDISK\n");
		fprintf(out, "  inname = \'%s\'\n", expcode);
		fprintf(out, "  inclass = \'%s\'\n", classcode);
		fprintf(out, "  inseq = 1\n");
		alldata();
		fprintf(out, "  docalib = 2\n");
		fprintf(out, "  gainuse = %d\n", tabver("CL"));
		fprintf(out, "  calsour = \'%s\'\'\n", source);
		if(tabver("BP") > 0)
		{
			fprintf(out, "  doband = 1\n");
			fprintf(out, "  bpver = %d\n", tabver("BP"));
		}
		else
		{
			fprintf(out, "  doband = -1\n");
		}
		fprintf(out, "  flagver = %d\n", tabver("FG"));
		fprintf(out, "  in2disk = AIPSDISK\n");
		fprintf(out, "  in2name = \'%s\'\n", source);
		fprintf(out, "  in2cl = \'%s\'\n", model);
		fprintf(out, "  in2seq = 0\n");
		fprintf(out, "  invers = 1\n");
		fprintf(out, "  smodel = 0\n");
		fprintf(out, "  bchan = 3\n");
		fprintf(out, "  echan = 30\n");
		fprintf(out, "  dofit = 0; antuse = 0; subarray = 0\n");
		fprintf(out, "  uvrange = 0, 0\n");
		fprintf(out, "  smooth = 0\n");
		fprintf(out, "  ncomp = 0\n");
		fprintf(out, "  nmaps = 1\n");
		fprintf(out, "  clro\n");
		fprintf(out, "  solmode = \'A&P\'\n");
		fprintf(out, "  solint = 15.0\n");
		fprintf(out, "  aparm = 4 0 0 0 0 1 3.7 0\n");
		fprintf(out, "  cparm = 0 1 0\n");
		sprintf(desc, "CALIB A&P %s\n", source);
		inctab("SN", desc);
		fprintf(out, "  snver = %d\n", tabver("SN"));
		fprintf(out, "GO\n");
		fprintf(out, "WAIT \'calib\'\n");
		
		fprintf(out, "\nTASK \'clcal\'\n");
		fprintf(out, "  indisk = AIPSDISK\n");
		fprintf(out, "  inname = \'%s\'\n", expcode);
		fprintf(out, "  inclass = \'%s\'\n", classcode);
		fprintf(out, "  inseq = 1\n");
		alldata();
		fprintf(out, "  calsour = \'%s\'\'\n", source);
		if(allsource) fprintf(out, "  sour = \'\'\n");
		else fprintf(out, "  sour = \'%s\'\'\n", source);
		fprintf(out, "  snver = %d\n", tabver("SN"));
		fprintf(out, "  inver = 0\n");
		fprintf(out, "  gainver = %d\n", tabver("CL"));
		sprintf(desc, "CLCAL calib A&P %s (CL%d x SN%d)", 
			source, tabver("CL"), tabver("SN"));
		inctab("CL", desc);
		fprintf(out, "  gainuse = %d\n", tabver("CL"));
		fprintf(out, "  opcode = \'\'\n");
		fprintf(out, "  interpol = \'\'\n");
		fprintf(out, "  intparm = 0\n");
		fprintf(out, "  smotype = \'\'\n");
		fprintf(out, "GO\n");
		fprintf(out, "WAIT \'clcal\'\n");
	}
}

void split(const char *source, int CLver, const char *outcl)
{
	fprintf(out, "\nTASK \'split\'\n");
	fprintf(out, "  indisk = AIPSDISK\n");
	fprintf(out, "  inname = \'%s\'\n", expcode);
	fprintf(out, "  inclass = \'%s\'\n", classcode);
	fprintf(out, "  inseq = 1\n");
	alldata();
	fprintf(out, "  source = \'%s\'\'\n", source);
	if(tabver("BP") > 0)
	{
		fprintf(out, "  doband = 1\n");
		fprintf(out, "  bpver = %d\n", tabver("BP"));
		fprintf(out, "  bchan = 2; echan = 31\n");
	}
	else
	{
		fprintf(out, "  doband = -1\n");
		fprintf(out, "  bchan = 4; echan = 29\n");
	}
	fprintf(out, "  flagver = %d\n", tabver("FG"));
	fprintf(out, "  docalib = 2\n");
	fprintf(out, "  gainuse = %d\n", CLver);
	fprintf(out, "  douvcomp = -1\n");
	fprintf(out, "  aparm = 2 0\n");
	fprintf(out, "  outdisk = AIPSDISK\n");
	if(outcl) fprintf(out, "  outcl = \'%s\'\n", outcl);
	else fprintf(out, "  outcl = \'SPLIT\'\n");
	fprintf(out, "  stokes = \'I\'\n");
	fprintf(out, "GO\n");
	fprintf(out, "WAIT \'split\'\n");
	fprintf(tablog, "SPLIT %s CL%d FG%d BP%d\n", source, CLver,
		tabver("FG"), tabver("BP"));
}

void image(const char *source, const char *inclass, int usebox)
{
	FILE *inbox, *outbox;
	char boxfilename[100], command[150];
	
	fprintf(out, "\nTASK \'imagr\'\n");
	fprintf(out, "  indisk = AIPSDISK\n");
	fprintf(out, "  inname = \'%s\'\n", source);
	fprintf(out, "  inclass = \'%s\'\n", inclass);
	fprintf(out, "  inseq = 0\n");
	alldata();
	fprintf(out, "  docalib = 0\n");
	fprintf(out, "  doband = 0\n");
	fprintf(out, "  bif = 1; eif = 0; bchan = 1; echan = 0\n");
	fprintf(out, "  nchav = 1000\n");
	fprintf(out, "  nfield = 1\n");
	fprintf(out, "  stokes = \'I\'\n");
	fprintf(out, "  outdisk = AIPSDISK\n");
	fprintf(out, "  outname = \'\'\n");
	fprintf(out, "  outseq = 0\n");
	fprintf(out, "  outvers = 1\n");
	fprintf(out, "  cellsize = 0.00075 0.00075\n");
	fprintf(out, "  imsiz = 512 512\n");
	fprintf(out, "  nfie = 1\n");
	fprintf(out, "  dotv = 1\n");
	fprintf(out, "  niter = 2000\n");
	if(usebox) 
	{
		fprintf(out, "  boxfile = \'%s:%s.box\'\n",
			boxfilelocation, source);
		sprintf(boxfilename, "%s/%s.BOX", boxfilepath, source);
		inbox = fopen(boxfilename, "r");
		if(inbox) fclose(inbox);
		else
		{
			outbox = fopen(boxfilename, "w");
			fprintf(outbox, "1    200    200    312    312\n");
			fclose(outbox);
			sprintf(command, "chmod a+w %s", boxfilename);
			system(command);
		}
	}
	else fprintf(out, "  boxfile = \'\'\n");
	fprintf(out, "  oboxfile = \'%s:%s.box\'\n", boxfilelocation, source);
	fprintf(out, "  robust = 3\n");
	fprintf(out, "  minpatch = 101\n");
	fprintf(out, "  imagrprm = 25 0\n");
	fprintf(out, "  clr2n\n");
	fprintf(out, "TVIN\n");
	fprintf(out, "GO\n");
	fprintf(out, "WAIT \'imagr\'\n");
	fprintf(out, "RECAT\n");
	fprintf(out, "  inname = \'%s\'\n", source);
	fprintf(out, "  inclass = \'ICL001\'\n");
	fprintf(out, "  inseq = 0\n");
	fprintf(out, "  inver = 1; outver = 2\n");
	fprintf(out, "  GO CCMRG ; WAIT CCMRG\n");
	fprintf(out, "  outn = \'\'\n");
	fprintf(out, "  outclass = \'I-----\'\n");
	fprintf(out, "  outseq = 0\n");
	fprintf(out, "RENAME\n");
}

void selfcal(const char *source, const char *inclass, const char *solmode,
	double solint)
{
	fprintf(out, "\nTASK \'calib\'\n");
	fprintf(out, "  indisk = AIPSDISK\n");
	fprintf(out, "  inname = \'%s\'\n", source);
	fprintf(out, "  inclass = \'%s\'\n", inclass);
	fprintf(out, "  inseq = 0\n");
	alldata();
	fprintf(out, "  docalib = 0\n");
	fprintf(out, "  gainuse = 0\n");
	fprintf(out, "  doband = 0\n");
	fprintf(out, "  bpver = 0\n");
	fprintf(out, "  calsour = \'%s\'\'\n", source);
	fprintf(out, "  in2disk = AIPSDISK\n");
	fprintf(out, "  in2name = \'%s\'\n", source);
	fprintf(out, "  in2class = \'I-----\'\n");
	fprintf(out, "  in2seq = 0\n");
	fprintf(out, "  invers = 0\n");
	fprintf(out, "  dofit = 0; antuse = 0; subarray = 0\n");
	fprintf(out, "  uvrange = 0, 0\n");
	fprintf(out, "  smooth = 0\n");
	fprintf(out, "  smodel = 0\n");
	fprintf(out, "  ncomp = 0\n");
	fprintf(out, "  nmaps = 1\n");
	fprintf(out, "  outdisk = AIPSDISK\n");
	fprintf(out, "  outname = \'\'\n");
	fprintf(out, "  outclass = \'XCALIB\'\n");
	fprintf(out, "  outseq = 0\n");
	fprintf(out, "  solmode = \'%s\'\n", solmode);
	fprintf(out, "  solint = %f\n", solint);
	fprintf(out, "  aparm = 4 0 0 0 0 1 3.7 0\n");
	fprintf(out, "  cparm = 0 1 0\n");
	fprintf(out, "  snver = 0\n");
	fprintf(out, "GO\n");
	fprintf(out, "WAIT \'calib\'\n");
}

void calib(const char *source, const char *inclass, const char *solmode, double solint)
{
	fprintf(out, "\nTASK \'calib\'\n");
	fprintf(out, "  indisk = AIPSDISK\n");
	fprintf(out, "  inname = \'%s\'\n", source);
	fprintf(out, "  inclass = \'%s\'\n", inclass);
	fprintf(out, "  inseq = 0\n");
	alldata();
	fprintf(out, "  docalib = 0\n");
	fprintf(out, "  gainuse = 0\n");
	fprintf(out, "  doband = 0\n");
	fprintf(out, "  bpver = 0\n");
	fprintf(out, "  calsour = \'%s\'\'\n", source);
	fprintf(out, "  clr2n\n");
	fprintf(out, "  smodel = 1 0\n");
	fprintf(out, "  dofit = 0; antuse = 0; subarray = 0\n");
	fprintf(out, "  uvrange = 0, 0\n");
	fprintf(out, "  smooth = 0\n");
	fprintf(out, "  ncomp = 0\n");
	fprintf(out, "  nmaps = 0\n");
	fprintf(out, "  outname = \'%s\'\n", source);
	fprintf(out, "  outclass = \'XCALIB\'\n");
	fprintf(out, "  outseq = 0\n");
	fprintf(out, "  solmode = \'%s\'\n", solmode);
	fprintf(out, "  solint = %f\n", solint);
	fprintf(out, "  aparm = 4 0 0 0 0 1 3.7 0\n");
	fprintf(out, "  cparm = 0 1 0\n");
	fprintf(out, "  snver = 0\n");
	fprintf(out, "GO\n");
	fprintf(out, "WAIT \'calib\'\n");
}

void imagecal(const char *source, const char *inclass, int nphase, int namp,
	double solint)
{
	int i;
	
	image(source, inclass, 1);
	for(i = 0; i < nphase; i++)
	{
		selfcal(source, inclass, "P", solint);
		image(source, "XCALIB", 1);
	}
	for(i = 0; i < namp; i++)
	{
		selfcal(source, inclass, "A&P", solint);
		image(source, "XCALIB", 1);
	}
	fprintf(out, "  indisk = AIPSDISK\n");
	fprintf(out, "  inname = \'%s\'\n", source);
	fprintf(out, "  inclass = \'I-----\'\n");
	fprintf(out, "  inseq = 0\n");
	fprintf(out, "  outname = \'\'\n");
	fprintf(out, "  outclass = \'%s\'\n", model);
	fprintf(out, "  outseq = 0\n");
	fprintf(out, "RENAME\n");
	
	fprintf(out, "  inclass = \'%s\'\n", model);
	fprintf(out, "  blc = 200, 200\n");
	fprintf(out, "  trc = 312, 312\n");
	fprintf(out, "  clev = 0.005\n");
	fprintf(out, "  levs = -2, -1, 1, 2, 4, 8, 16, 32, 64, 128, 256\n");
	fprintf(out, "  inext = \'CC\'\n");
	fprintf(out, "  invers = 0\n");
	fprintf(out, "  dotv = 1\n");
	fprintf(out, "  tvin\n");
	fprintf(out, "GO CCNTR ; WAIT CCNTR\n");
}

void calimagecal(const char *source, const char *inclass, int nphase, int namp,
	double solint)
{
	int i;
	
	calib(source, inclass, "P", solint);
	image(source, "XCALIB", 1);
	for(i = 0; i < nphase; i++)
	{
		selfcal(source, inclass, "P", solint);
		image(source, "XCALIB", 1);
	}
	for(i = 0; i < namp; i++)
	{
		selfcal(source, inclass, "A&P", solint);
		image(source, "XCALIB", 1);
	}
	fprintf(out, "  indisk = AIPSDISK\n");
	fprintf(out, "  inname = \'%s\'\n", source);
	fprintf(out, "  inclass = \'I-----\'\n");
	fprintf(out, "  inseq = 0\n");
	fprintf(out, "  outname = \'\'\n");
	fprintf(out, "  outclass = \'%s\'\n", model);
	fprintf(out, "  outseq = 0\n");
	fprintf(out, "RENAME\n");
}

void bpass(const char *source)
{
	char desc[1000];

	fprintf(out, "\nTASK \'bpass\'\n");
	fprintf(out, "  indisk = AIPSDISK\n");
	fprintf(out, "  inname = \'%s\'\n", expcode);
	fprintf(out, "  inclass = \'%s\'\n", classcode);
	fprintf(out, "  inseq = 1\n");
	alldata();
	fprintf(out, "  docalib = 2\n");
	fprintf(out, "  gainuse = %d\n", tabver("CL"));
	fprintf(out, "  doband = 0\n");
	fprintf(out, "  calsour = \'%s\'\'\n", source);
	fprintf(out, "  in2disk = AIPSDISK\n");
	fprintf(out, "  in2name = \'%s\'\n", source);
	fprintf(out, "  in2class = \'%s\'\n", model);
	fprintf(out, "  in2seq = 0\n");
	fprintf(out, "  invers = 1\n");
	fprintf(out, "  outvers = %d\n", tabver("BP")+1);
	fprintf(out, "  smodel = 0\n");
	fprintf(out, "  solint = -1\n");
	fprintf(out, "  flagver = %d\n", tabver("FG"));
	fprintf(out, "  bpassprm = 0\n");
	fprintf(out, "  smooth = 1, 4, 4\n");
	fprintf(out, "  clr3n\n");
	fprintf(out, "GO\n");
	fprintf(out, "WAIT \'bpass\'\n");
	sprintf(desc, "BPASS %s", source);
	inctab("BP", desc);
	
	fprintf(out, "\nTASK \'possm\'\n");
	fprintf(out, "  indisk = AIPSDISK\n");
	fprintf(out, "  inname = \'%s\'\n", expcode);
	fprintf(out, "  inclass = \'%s\'\n", classcode);
	fprintf(out, "  inseq = 1\n");
	alldata();
	fprintf(out, "  docalib = -1\n");
	fprintf(out, "  doband = 1\n");
	fprintf(out, "  bpver = %d\n", tabver("BP"));
	fprintf(out, "  sour = \'\'\n");
	fprintf(out, "  calsour = \'\'\n");
	fprintf(out, "  smooth = 0\n");
	fprintf(out, "  clr2n\n");
	fprintf(out, "  aparm 0 0 0 0 0 0 0 2 1 0\n");
	fprintf(out, "  dotv = 1\n");
	fprintf(out, "  bparm 0\n");
	fprintf(out, "  nplots = 2\n");
	fprintf(out, "  solint = 500\n");
	fprintf(out, "TVIN\n");
	fprintf(out, "GO\n");
	fprintf(out, "WAIT \'possm\'\n");
}

void uvflg(const char *flag, int logit)
{
	char desc[1000];

	fprintf(out, "\nTASK \'uvflg\'\n");
	fprintf(out, "  indisk = AIPSDISK\n");
	fprintf(out, "  inname = \'%s\'\n", expcode);
	fprintf(out, "  inclass = \'%s\'\n", classcode);
	fprintf(out, "  inseq = 1\n");
	fprintf(out, "  infile = \'\'\n");
	fprintf(out, "  opcode = \'\'\n");
	fprintf(out, "  bdrop = 0 ; edrop = 0\n");
	alldata();
	fprintf(out, "  source = \'\'\n");
	fprintf(out, "  flagver = %d\n", tabver("FG"));
	fprintf(out, "  aparm = 0\n");
	fprintf(out, "  %s\n",flag);
	fprintf(out, "GO\n");
	fprintf(out, "WAIT \'uvflg\'\n");
	if(logit)
	{
		sprintf(desc, "UVFLG %s", flag);
		modtab("FG", desc);
	}
}

void uvmln()
{
	fprintf(out, "\nTASK \'uvmln\'\n");
	fprintf(out, "  indisk = AIPSDISK\n");
	fprintf(out, "  inname = \'%s\'\n", expcode);
	fprintf(out, "  inclass = \'%s\'\n", classcode);
	fprintf(out, "  inseq = 1\n");
	alldata();
	fprintf(out, "  sour = \'\'\'\n");
	if(tabver("BP") > 0)
	{
		fprintf(out, "  doband = 1\n");
		fprintf(out, "  bpver = %d\n", tabver("BP"));
	}
	else
	{
		fprintf(out, "  doband = -1\n");
	}
	fprintf(out, "  flagver = %d\n", tabver("FG"));
	fprintf(out, "  docalib = 0\n");
	fprintf(out, "  order = 1\n");
	fprintf(out, "  flux = 0\n");
	fprintf(out, "  ichansel = 0\n");
	fprintf(out, "GO\n");
	fprintf(out, "WAIT \'uvmln\'\n");
	modtab("FG", "uvmln");
}

void calcopy(const char *inext, int inver)
{
	fprintf(out, "  \nTASK \'tacop\'\n");
	fprintf(out, "    indisk = AIPSDISK\n");
	fprintf(out, "    inname = \'%s\'\n", expcode);
	fprintf(out, "    inclass = CALXFR\n");
	fprintf(out, "    inseq = 1\n");
	fprintf(out, "    inext = \'%s\'\n", inext);
	fprintf(out, "    inver = %d\n", inver);
	fprintf(out, "    ncount = 1\n");
	fprintf(out, "    outdisk = AIPSDISK\n");
	fprintf(out, "    outname = \'%s\'\n", expcode);
	fprintf(out, "    outclass = \'%s\'\n", classcode);
	fprintf(out, "    outseq = 1\n");
	fprintf(out, "    keyword = \'\'\n");
	fprintf(out, "    keystrng = \'\'\n");
	fprintf(out, "    keyvalue = 0\n");
	fprintf(out, "    outvers = %d\n", inver);
	fprintf(out, "  GO\n");
	fprintf(out, "  WAIT \'tacop\'\n");
}

void calreplace(const char *inext, int inver)
{
	fprintf(out, "\n");
	fprintf(out, "    indisk = AIPSDISK\n");
	fprintf(out, "    inname = \'%s\'\n", expcode);
	fprintf(out, "    inclass = \'%s\'\n", classcode);
	fprintf(out, "    inseq = 1\n");
	fprintf(out, "    inext = \'%s\'\n", inext);
	fprintf(out, "    inver = %d\n", inver);
	fprintf(out, "  EXTD\n");
	fprintf(out, "  TASK \'tacop\'\n");
	fprintf(out, "    indisk = AIPSDISK\n");
	fprintf(out, "    inname = \'%s\'\n", expcode);
	fprintf(out, "    inclass = CALXFR\n");
	fprintf(out, "    inseq = 1\n");
	fprintf(out, "    inext = \'%s\'\n", inext);
	fprintf(out, "    inver = %d\n", inver);
	fprintf(out, "    ncount = 1\n");
	fprintf(out, "    outdisk = AIPSDISK\n");
	fprintf(out, "    outname = \'%s\'\n", expcode);
	fprintf(out, "    outclass = \'%s\'\n", classcode);
	fprintf(out, "    outseq = 1\n");
	fprintf(out, "    keyword = \'\'\n");
	fprintf(out, "    keystrng = \'\'\n");
	fprintf(out, "    keyvalue = 0\n");
	fprintf(out, "    outvers = %d\n", inver);
	fprintf(out, "  GO\n");
	fprintf(out, "  WAIT \'tacop\'\n");
}

FILE *newfile(const char *ext)
{
	char buf[1000];
	time_t t;

	t = time(0);

	sprintf(buf, "%s/%s.001", aapath, ext);
	fprintf(stderr, "writing to %s\n", buf);
	out = fopen(buf, "w");
	if(!out) 
	{
		fprintf(stderr, "Error opening %s for write\n", buf);
		return 0;
	}
	fprintf(out, "* %s.001\n", ext);
	fprintf(out, "* Autogenerated on %s* with autoaips version %s\n",
		ctime(&t), version);
	fprintf(out, "*\n* Make sure that you type \" version = \'%s\' \"\n",
		aalocation);
	fprintf(out, "* and then run this with \" RUN %s \"\n", ext);
	fprintf(out, "\n");
	fprintf(out, "restore 0\n");
	fprintf(out, "PRTASK = \'\'\n");
	fprintf(out, "PRTIME = 0\n");
	fprintf(out, "PRNUMBER = 0\n");
	fprintf(out, "CLRMSG\n");
	fprintf(out, "version = \'%s\'\n", aalocation);
	fprintf(out, "PROC STOOPID\n");
	fprintf(out, "  SCALAR AIPSDISK\n");
	fprintf(out, "  SCALAR REF_ANT\n");
	fprintf(out, "  STRING*6 CALXFR\n");
	fprintf(out, "FINISH\n");
	fprintf(out, "PROC cleanup\n");
	fprintf(out, "  PRINT \'Cleaning up...\'\n");
	fprintf(out, "  indisk = AIPSDISK\n");
	fprintf(out, "  recat\n");
	fprintf(out, "  clrmsg\n");
	fprintf(out, "  for i = 1 to 2000\n");
	fprintf(out, "    egetname i\n");
	fprintf(out, "    if error then i = 2000; else\n");
	fprintf(out, "      clrst\n");
	fprintf(out, "      if substr(incl, 1, 6) = \'XCALIB\'\n");
	fprintf(out, "        then zap\n");
	fprintf(out, "      end\n");
	fprintf(out, "      if substr(incl, 1, 5) = \'I----\'\n");
	fprintf(out, "        then zap\n");
	fprintf(out, "      end\n");
	fprintf(out, "      if substr(incl, 1, 5) = \'UVTMP\'\n");
	fprintf(out, "        then zap\n");
	fprintf(out, "      end\n");
	fprintf(out, "      if substr(incl, 1, 4) = \'IBM0\'\n");
	fprintf(out, "        then zap\n");
	fprintf(out, "      end\n");
	fprintf(out, "    end\n");
	fprintf(out, "  end\n");
	fprintf(out, "  recat\n");
	fprintf(out, "  return\n");
	fprintf(out, "FINISH\n");
	fprintf(out, "PROC uvcleanup\n");
	fprintf(out, "  PRINT \'Cleaning up...\'\n");
	fprintf(out, "  indisk = AIPSDISK\n");
	fprintf(out, "  recat\n");
	fprintf(out, "  clrmsg\n");
	fprintf(out, "  for i = 1 to 2000\n");
	fprintf(out, "    egetname i\n");
	fprintf(out, "    if error then i = 2000; else\n");
	fprintf(out, "      clrst\n");
	fprintf(out, "      if substr(incl, 1, 4) = \'XPSR\'\n");
	fprintf(out, "        then zap\n");
	fprintf(out, "      end\n");
	fprintf(out, "      if substr(incl, 1, 3) = \'XIB\'\n");
	fprintf(out, "        then zap\n");
	fprintf(out, "      end\n");
	fprintf(out, "    end\n");
	fprintf(out, "  end\n");
	fprintf(out, "  recat\n");
	fprintf(out, "  return\n");
	fprintf(out, "FINISH\n");
	fprintf(out, "*\n* Change the next line to change the aips disk\n");
	fprintf(out, "AIPSDISK = %d\n", disk);
	fprintf(out, "*\n");
	fprintf(out, "* Change the next line to change the reference antenna\n");
	fprintf(out, "REF_ANT = %d\n", refant);
	fprintf(out, "*\n*\n");
	fprintf(out, "outdisk = AIPSDISK; indisk = AIPSDISK\n");
	fprintf(out, "baddisk = AIPSDISK, 0\n");
	fprintf(out, "CALXFR = \'IB1\'\n");
	fprintf(out, "clr2n\n");
	fprintf(out, "recat\n\n");

	return out;
}	

void resetparams(const char *sour)
{
	fprintf(out, "\n");
	fprintf(out, "TGET imagr\n");
	fprintf(out, "  inname = \'%s\'\n", sour);
	fprintf(out, "  inclass = \'SPLIT1\'\n");
	fprintf(out, "  inseq = 0\n");
	fprintf(out, "  sour = \'\'\n");
	fprintf(out, "  outname = \'\'\n");
	fprintf(out, "  boxfile = \'\'\n");
	fprintf(out, "  oboxfile = \'\'\n");
	fprintf(out, "  bif = 1; eif = 0; bchan = 1; echan = 0\n");
	fprintf(out, "  nchav = 1000\n");
	fprintf(out, "  nfield = 1\n");
	fprintf(out, "  docalib = -1\n");
	fprintf(out, "  cellsize = 0.0012, 0.0012\n");
	fprintf(out, "  dotv = 1\n");
	fprintf(out, "TPUT imagr\n");
	fprintf(out, "\n");
	fprintf(out, "TGET calib\n");
	alldata();
	fprintf(out, "  inname = \'%s\'\n", sour);
	fprintf(out, "  inclass = \'SPLIT1\'\n");
	fprintf(out, "  inseq = 0\n");
	fprintf(out, "  calsour = \'\'\n");
	fprintf(out, "  outdisk = AIPSDISK\n");
	fprintf(out, "  outname = \'\'\n");
	fprintf(out, "  outclass = \'XCALIB\'\n");
	fprintf(out, "  solint = 2.0\n");
	fprintf(out, "  solmode = \'P\'\n");
	fprintf(out, "  docalib = -1\n");
	fprintf(out, "  doband = -1\n");
	fprintf(out, "  in2disk = AIPSDISK\n");
	fprintf(out, "  in2name = \'%s\'\n", sour);
	fprintf(out, "  in2class = \'\'\n");
	fprintf(out, "  in2seq = 0\n");
	fprintf(out, "  smodel = 0\n");
	fprintf(out, "  aparm(7) = 3\n");
	fprintf(out, "TPUT calib\n");
	fprintf(out, "\n");
	fprintf(out, "TASK \'tacop\'\n");
	fprintf(out, "  inver = 0\n");
	fprintf(out, "  inext = \'SN\'\n");
	fprintf(out, "  ncount = 1\n");
	fprintf(out, "  outver = 0\n");
	fprintf(out, "  ncount = 1\n");
	fprintf(out, "TPUT tacop\n");
	fprintf(out, "\n");
}

int main(int argc, char **argv)
{
	wordexp_t wordbuf;
	char globbase[100], buf[1000], buf2[1000], str[10];
	int i, j, l, C, S;
	int mainphasecalcl = 0;
	const char *phasecal, *pulsar, *geod, *flag;
	char target[100] = "", outname[100];
	FILE *pin, *inas, *outas;
	int epoch = 0;
	int pcalsn[10];

	if(argc < 4) return usage(argv[0]);

	disk = atoi(argv[3]);
	if(disk < 1)
	{
		fprintf(stderr, "disk needs to be > 0\n");
		return 0;
	}
	fprintf(stderr, "disk : %d\n", disk);

	aapath = getenv(argv[1]);
	if(!aapath)
	{
		fprintf(stderr, "** Error! Env. Var. %s not defined!\n",
			argv[1]);
		return 0;
	}
	printf(" env = %s\n", aapath);
	strcpy(classcode, argv[2]);
	strcpy(aalocation, argv[1]);

	if(strlen(classcode) != 3)
	{
		fprintf(stderr, "Class code (argument 2) needs to be one of\n"
				"  psr, ib1, ib2, ib3, ...\n");
		return 0;
	}

	if(classcode[0] == 'i' && classcode[1] == 'b')
	{
		pass = atoi(classcode+2);
		if(pass < 1 || pass > 8)
		{
		fprintf(stderr, "Class code (argument 2) needs to be one of\n"
				"  psr, ib1, ib2, ib3, ...\n");
		return 0;
		}
	}
	else 
	{
		if(strcmp(classcode, "psr") == 0) pass = 0;
		else
		{
		fprintf(stderr, "Class code (argument 2) needs to be one of\n"
				"  psr, ib1, ib2, ib3, ...\n");
		return 0;
		}
	}

	if(getenv("IONEXPY")) strcpy(ionexpy, getenv("IONEXPY"));
	else strcpy(ionexpy, "ionex.py");

	l = strlen(aapath);

	sprintf(globbase, "%s/*.x.?", aapath);
	fprintf(stderr, "Looking for %s\n", globbase);
	wordbuf.we_offs = 0;
	i = wordexp(globbase, &wordbuf, 0);
	if(wordbuf.we_wordc < 1 || !strcmp(globbase, wordbuf.we_wordv[0]))
	{
		fprintf(stderr, "** Warning : no <pulsar>.x.n was found.  "
				"Looking for <pulsar>.x\n");
		wordfree(&wordbuf);
		sprintf(globbase, "%s/*.x", aapath);
	fprintf(stderr, "Looking for %s\n", globbase);
		wordbuf.we_offs = 0;
		epoch = -1;
		wordexp(globbase, &wordbuf, 0);
		if(wordbuf.we_wordc < 1 || 
			!strcmp(globbase, wordbuf.we_wordv[0]))
		{
			wordfree(&wordbuf);
			fprintf(stderr, "** Error : no .x file found\n");
			return 0;
		}
	}
	if(wordbuf.we_wordc > 1)
	{
		fprintf(stderr, "** Warning : using first of multiple "
				"candidate .x files\n");
	}
	fprintf(stderr, "Using .x file : %s\n", wordbuf.we_wordv[0]);
	kv = loadKeyValue(wordbuf.we_wordv[0]);
	if(!kv)
	{
		fprintf(stderr, "** Error : cannot open %s as a .x file\n",
			wordbuf.we_wordv[0]);
		return 0;
	}
	if(epoch >= 0)
	{
		l = strlen(wordbuf.we_wordv[0]);
		for(i = l-1; i >= 0; i--) 
			if(wordbuf.we_wordv[0][i] == '.') break;
		if(i == 0) epoch = -1;
		else 
		{
			epoch = atoi(wordbuf.we_wordv[0]+i+1);
			if(epoch <= 0 || epoch > 100)
			{
				epoch = -1;
				fprintf(stderr, "** Please rename the .x "
						"file as .x.N where N is the "
						"epoch number\n");
				fprintf(stderr, "** Warning : Cannot "
						"determine epoch\n");
			}
			else fprintf(stderr, "Looks like epoch %d\n", epoch);
		}
	}
	wordfree(&wordbuf);

	refant = getKeyValueint(kv, "refant");
	if(refant < 0) refant = 4;

	pulsar = getKeyValuestring(kv, "name");
	if(!pulsar)
	{
		fprintf(stderr, "** Error : pulsar name not found in .x"
				" file\n");
		return 0;
	}

	sprintf(globbase, "%s/*.key", aapath);
	wordbuf.we_offs = 0;
	wordexp(globbase, &wordbuf, 0);
	fprintf(stderr, "Using .key file : %s\n", wordbuf.we_wordv[0]);
	l = strlen(aapath);
	strcpy(keyfile, wordbuf.we_wordv[0]+l+1);
	strcpy(expcode, keyfile);
	for(i = 0; expcode[i]; i++) if(expcode[i] == '.') expcode[i] = 0;
	fprintf(stderr, "expcode = %s\n", expcode);
	wordfree(&wordbuf);

	if(epoch < 1)
	{
		strcpy(str, expcode);
		str[5] = 0;
		if(strcmp(str, "bc123") == 0)
		{
			if(strlen(expcode) < 7) epoch = 1;
			else if(expcode[5] == 'a')
			{
				if(expcode[6] < 'i') epoch = 1;
				else epoch = 2;
			}
		}
		if(epoch > 0) fprintf(stderr, "** Warning : Assuming this is "
					      "epoch %d\n", epoch);
	}

	if(epoch < 1) 
	{
		sprintf(model,   "MODEL");
		sprintf(geodext, "GEOD");
	}
	else 
	{
		sprintf(model,    "MODEL%d", epoch);
		sprintf(geodext,  "GEOD%d",  epoch);
	}

	if(argc > 4) strcpy(datalocation, argv[4]);
	else
	{
		if(getenv("DATAFILES"))
			strcpy(datalocation, "DATAFILES");
		else strcpy(datalocation, argv[1]);
	}

	datapath = getenv(datalocation);
	if(!datapath)
	{
		fprintf(stderr, "** Error : cannot expand env. var. %s for data location\n", datalocation);
		return 0;
	}

	if(getenv("BOXFILES"))
	{
		strcpy(boxfilelocation, "BOXFILES");
		strcpy(boxfilepath, getenv("BOXFILES"));
	}

	if(argc > 5)
		strcpy(boxfilelocation, argv[5]);
	else
		strcpy(boxfilelocation, argv[1]);

	boxfilepath = getenv(boxfilelocation);
	if(!boxfilepath)
	{
		fprintf(stderr, "** Error : cannot expand Env. Var. %s for " 
				"boxfile location\n", boxfilelocation);
		return 0;
	}
	
	bpasscal = getKeyValuestring(kv, "bpass_cal_name");
	if(bpasscal == 0) bpasscal = getKeyValuestring(kv, "phase_cal_name1");
	if(bpasscal == 0)
	{
		fprintf(stderr, "** Error : cannot find bandpass calibrator\n");
		return 0;
	}
	
	out = newfile("XLOAD");
	init();

	sprintf(globbase, "%s/*.FITS", datapath);
	wordbuf.we_offs = 0;
	wordexp(globbase, &wordbuf, 0);
	if(wordbuf.we_wordc < 1 || !strcmp(globbase, wordbuf.we_wordv[0]))
	{
		fprintf(stderr, "** Error : no FITS files found in %s\n",
				datapath);
	}
	l = strlen(datapath);
	for(i = 0; i < wordbuf.we_wordc; i++) 
	{
		printf("  %s\n", wordbuf.we_wordv[i]+l+1);
		sprintf(buf, "%s:%s", datalocation, wordbuf.we_wordv[i]+l+1);
		fitld(buf);
		strcpy(buf2, pulsar);
		buf2[8] = 0;
		if(strlen(target) < 4)
		{
			sprintf(buf, "%s/autoaips.save", aapath);
			inas = fopen(buf, "r");
			if(inas)
			{
				fscanf(inas, "%s", target);
				fclose(inas);
			}
			else
			{
				sprintf(buf, "strings %s | grep %s", 
					wordbuf.we_wordv[i], buf2);
				fprintf(stderr, "Executing : %s\n", buf);
				pin = popen(buf, "r");
				fgets(buf, 999, pin);
				if(feof(pin))
				{
					fclose(pin);
					continue;
				}
				fclose(pin);
				fprintf(stderr, "output line : %s\n", buf);
				for(j = 0; buf[j] >= ' '; j++);
				buf[j] = 0;
				strcpy(target, buf);
				fprintf(stderr, "Target name in FITS files : %s\n",
					target);
				sprintf(buf, "%s/autoaips.save", aapath);
				outas = fopen(buf, "w");
				fprintf(outas, "%s", target);
				fclose(outas);
			}
		}
	}
	if(strlen(target) < 4)
	{
		fprintf(stderr, "Target not found in fits files\n");
		return 0;
	}
	if(pass == 0) strcpy(outname, pulsar);
	else sprintf(outname, "%s.%d", pulsar, pass);
	fprintf(stderr, "outname = %s\n", outname);
	wordfree(&wordbuf);

	indxr();
	
	fprintf(out, "* \'Data loaded.  "
		"Please check for subarray conditions.\'\n");
	fprintf(out, "* \'(If INDXR ran successfully, then you are safe.)\'\n");
	fprintf(out, "* \'When satisfied with data, run XAPRIORI.\'\n");
	fclose(out);
	out = newfile("XAPRIORI");

	fxpol();
	mergecal();
	accor();
	apcal();
	clcor(0);
	if(tecor(0) < 0) 
	{
		
		fprintf(stderr, "** Error : LST days don't match in .x and "
				".key files\n");
		return 0; 
	}

	/* apriori flagging */
	
	aprioriCL = tabver("CL");
	aprioriSN = tabver("SN");

	for(i = 1; i < 100; i++)
	{
		sprintf(buf, "flag%d", i);
		flag = getKeyValuestring(kv, buf);
		if(flag) uvflg(flag, 1);
	}

	setuptbout(argv[1]);
	setupspflg(bpasscal);

	fprintf(out, "* \'A-priori calibration done.\'\n");
	fprintf(out, "* \'Now flag the data.\'\n");
	fprintf(out, "* \'You can start SPFLG by typing \"GO\".\'\n");
	fprintf(out, "* \'Put flag commands in the .x file, rerun autoaips,\'\n");
	fprintf(out, "* \'and then RUN XUVFLG to apply.\'\n");
	fprintf(out, "* \'Then run XBPASS to solve for bandpass,\'\n");
	fprintf(out, "* \'Or run XCOPY if In-beam #1 has been processed.\'\n");
	fclose(out);

	out = newfile("XUVFLG");
	for(i = 1; i < 999; i++)
	{
		sprintf(buf, "flag%d", i);
		flag = getKeyValuestring(kv, buf);
		if(flag) uvflg(flag, 0);
		else break;
	}
	fprintf(out, "* \'%d flags applied.  Now RUN XBPASS or\'\n", i-1);
	fprintf(out, "* \'RUN XCOPY if inbeam #1 is done.\'\n");
	
	fclose(out);


	out = newfile("XBPASS");
	
	fring(bpasscal, 0, 1.5, 0);
	split(bpasscal, tabver("CL"), "UVTMP");
	imagecal(bpasscal, "UVTMP", 0, 3, 1.5);
	fring(bpasscal, 1, 1.5, 0);
	bpass(bpasscal);

	fprintf(out, "* \'Bandpass calibration complete.  "
				"Please check results.\'\n");
	fprintf(out, "* \'Then run XPHASE for phase calibration.\'\n");
	fclose(out);

	S = tabver("SN");
	C = tabver("CL");
	out = newfile("XPHASE");

	for(i = 1; i <= 3; i++)
	{
		sprintf(buf, "phase_cal_name%d", i);
		phasecal = getKeyValuestring(kv, buf);
		if(!phasecal) break;
		if(strcmp(phasecal, bpasscal) != 0)
		{
			fring(phasecal, 0, 2.5, 0);
			split(phasecal, tabver("CL"), "UVTMP");
			imagecal(phasecal, "UVTMP", 1, 2, 1.5);
		}
		fring(phasecal, 1, 2.5, 1);
		pcalsn[i-1] = tabver("SN");
		split(phasecal, tabver("CL"), 0);
		split(target, tabver("CL"), 0);
		fprintf(out, "  indisk = AIPSDISK\n");
		fprintf(out, "  inname = \'%s\'\n", target);
		fprintf(out, "  inclass = \'SPLIT\'\n");
		fprintf(out, "  inseq = 0\n");
		fprintf(out, "  outname = \'%s\'\n", outname);
		fprintf(out, "  outclass = \'SPLIT%d\'\n", i);
		fprintf(out, "  outseq = 0\n");
		fprintf(out, "RENAME\n");
		if(mainphasecalcl == 0)
			mainphasecalcl = tabver("CL");
	}
	
	resetparams(outname);

	fprintf(out, "* \'XPHASE done.  Single source files created.\'\n");
	fclose(out);

	out = newfile("XCOPY");
	calcopy("BP", 1);
	calreplace("FG", 1);

	C = aprioriCL;

	for(i = 1; i <= 3; i++)
	{
		sprintf(buf, "phase_cal_name%d", i);
		phasecal = getKeyValuestring(kv, buf);
		if(!phasecal) break;
		
		calcopy("SN", pcalsn[i-1]-1);
	
		fprintf(out, "\nTASK \'clcal\'\n");
		fprintf(out, "  indisk = AIPSDISK\n");
		fprintf(out, "  inname = \'%s\'\n", expcode);
		fprintf(out, "  inclass = \'%s\'\n", classcode);
		fprintf(out, "  inseq = 1\n");
		alldata();
		fprintf(out, "  calsour = \'%s\'\'\n", phasecal);
		fprintf(out, "  sour = \'\'\n");
		fprintf(out, "  snver = %d\n", pcalsn[i-1]-1);
		fprintf(out, "  inver = 0\n");
		fprintf(out, "  gainver = %d\n", aprioriCL);
		fprintf(out, "  gainuse = %d\n", ++C);
		fprintf(out, "  opcode = \'\'\n");
		fprintf(out, "  interpol = \'ambg\'\n");
		fprintf(out, "  intparm = 0\n");
		fprintf(out, "  smotype = \'\'\n");
		fprintf(out, "GO\n");
		fprintf(out, "WAIT \'clcal\'\n");

		calcopy("SN", pcalsn[i-1]);
		
		fprintf(out, "\nTASK \'clcal\'\n");
		fprintf(out, "  indisk = AIPSDISK\n");
		fprintf(out, "  inname = \'%s\'\n", expcode);
		fprintf(out, "  inclass = \'%s\'\n", classcode);
		fprintf(out, "  inseq = 1\n");
		alldata();
		fprintf(out, "  calsour = \'%s\'\'\n", phasecal);
		fprintf(out, "  sour = \'\'\n");
		fprintf(out, "  snver = %d\n", pcalsn[i-1]);
		fprintf(out, "  inver = 0\n");
		fprintf(out, "  gainver = %d\n", C);
		fprintf(out, "  gainuse = %d\n", ++C);
		fprintf(out, "  opcode = \'\'\n");
		fprintf(out, "  interpol = \'\'\n");
		fprintf(out, "  intparm = 0\n");
		fprintf(out, "  smotype = \'\'\n");
		fprintf(out, "GO\n");
		fprintf(out, "WAIT \'clcal\'\n");
		
		split(target, C, 0);

		fprintf(out, "  indisk = AIPSDISK\n");
		fprintf(out, "  inname = \'%s\'\n", target);
		fprintf(out, "  inclass = \'SPLIT\'\n");
		fprintf(out, "  inseq = 0\n");
		fprintf(out, "  outname = \'%s\'\n", outname);
		fprintf(out, "  outclass = \'SPLIT%d\'\n", i);
		fprintf(out, "  outseq = 0\n");
		fprintf(out, "RENAME\n");
		if(mainphasecalcl == 0)
			mainphasecalcl = tabver("CL");
	}
	
	resetparams(outname);
	
	fprintf(out, "* \'XCOPY done.  Single source files created.\'\n");
	
	fclose(out);

	out = newfile("XGEOD");

	for(i = 1; i <= 8; i++)
	{
		sprintf(buf, "geod_cal_name%d", i);
		geod = getKeyValuestring(kv, buf);
		if(!geod) break;
		split(geod, mainphasecalcl, geodext);
		calimagecal(geod, geodext, 1, 1, 0.3);
		selfcal(geod, geodext, "A&P", 0.1);
	}
	
	fprintf(out, "* \'XGEOD done.  Single source geod files created.\'\n");
	fclose(out);
	
	
	if(tablog) fclose(tablog);

	getlog();

	printf("autoaips version %s finished\n", version);

	return 0;
}
